c ******************************************************* c
c
      subroutine calc_renormalized_pk(itype, ijob, zred)
c
c ******************************************************* c
c
c     computing renormalized power spectra up to 1-loop order 
c     using gaussian quadrature method
c
      implicit none
c
      integer itype, ijob, ialpha
      integer ik, ik_init, ikmax, ik_max1, ik_max2
      integer ix, ixmax
      parameter(ikmax=5000)
      parameter(ixmax=400)
      real*8  ak1(ikmax), ak2(ikmax), pk(ikmax) 
      real*8  pk_A11(ikmax), pk_A12(ikmax), pk_A22(ikmax)
      real*8  pk_B11(ikmax), pk_B12(ikmax), pk_B22(ikmax)
      real*8  zinit, zred, G11, G12, G21, G22
      real*8  growth, pi, pklin_kx
      real*8  k, kmin, kmax, xmin, xmax
      real*8  intB11, intB12, intB22
      real*8  wx(ixmax), xx(ixmax)
      parameter(zinit=500.d0)
      common /pk_data/ ak1, pk, ik_max1
      common /wave_number/  k, xmin, xmax
      common /alpha_corr/ ialpha
c     -------------------------------------------------
c
      pi = 4.d0 * datan(1.d0)
      ik_max2 = ik_max1
c
      if(ijob.eq.0) then
         call search(itype,ik_init)
         if(itype.eq.0 .and. ialpha.eq.1) 
     &        open(10,file='renormalized_pk2_RPT.dat',status='unknown',
     &        access='append')
         if(itype.eq.1 .and. ialpha.eq.1) 
     &    open(10,file='renormalized_pk2_Closure.dat',status='unknown',
     &    access='append')
         if(itype.eq.0 .and. ialpha.eq.0) 
     &        open(10,file='renormalized_pk2_RPT_alpha.dat',
     &        status='unknown', access='append')
         if(itype.eq.1 .and. ialpha.eq.0) 
     &    open(10,file='renormalized_pk2_Closure_alpha.dat',
     &        status='unknown', access='append')
      elseif(ijob.eq.1) then 
         ik_init = 1
         if(itype.eq.0 .and. ialpha.eq.1 ) 
     &        open(10,file='renormalized_pk2_RPT.dat',status='unknown',
     &        access='sequential')
         if(itype.eq.1 .and. ialpha.eq.1) 
     &    open(10,file='renormalized_pk2_Closure.dat',status='unknown',
     &    access='sequential')
         if(itype.eq.0 .and. ialpha.eq.0 ) 
     &        open(10,file='renormalized_pk2_RPT_alpha.dat',
     &        status='unknown', access='sequential')
         if(itype.eq.1 .and. ialpha.eq.0) 
     &    open(10,file='renormalized_pk2_Closure_alpha.dat',
     &        status='unknown', access='sequential')
      endif
c
c     //////// calculation of A-part P(k) ////////
c
      do ik=ik_init, ik_max1
c
         ak2(ik) = ak1(ik)
c
         if(itype.eq.0) then 
            call G_ab_RPT(ak1(ik), dlog(growth(zred)), 
     &           dlog(growth(zinit)), G11, G12, G21, G22)
         elseif(itype.eq.1) then
            call G_ab_Closure(ak1(ik), dlog(growth(zred)), 
     &           dlog(growth(zinit)), G11, G12, G21, G22)
         endif
         pk_A11(ik) = (G11 + G12) * (G11 + G12) * pk(ik) 
         pk_A11(ik) = pk_A11(ik) * growth(zinit)**2
         pk_A12(ik) = (G11 + G12) * (G21 + G22) * pk(ik) 
         pk_A12(ik) = pk_A12(ik) * growth(zinit)**2
         pk_A22(ik) = (G21 + G22) * (G21 + G22) * pk(ik) 
         pk_A22(ik) = pk_A22(ik) * growth(zinit)**2
      enddo
c
c     //////// calculation of B-part P(k) ////////c
c
      kmin = ak1(1)
      kmax = ak1(ik_max1)
c
      do 10 ik=ik_init, ik_max1
c
         k = ak1(ik)
c
         xmin = kmin / k
         xmax = kmax / k

         pk_B11(ik) = 0.0d0
         pk_B12(ik) = 0.0d0
         pk_B22(ik) = 0.0d0
c
c     skip the integration for negligible contribution from small-k 
c
         if(ak1(ik).lt.0.01) goto 9
c
c     integration by Gaussian quadratures method
c
         call gauleg(log(xmin), log(xmax), xx, wx, ixmax)
c
         do ix=1, ixmax
            xx(ix) = dexp(xx(ix))
            call find_pk(k*xx(ix), pklin_kx)
            call int_pk_B(xx(ix), zred, itype, intB11, intB12, intB22) 
            pk_B11(ik) = pk_B11(ik) + wx(ix) * pklin_kx * intB11
            pk_B12(ik) = pk_B12(ik) + wx(ix) * pklin_kx * intB12
            pk_B22(ik) = pk_B22(ik) + wx(ix) * pklin_kx * intB22
         enddo
c
         pk_B11(ik) = pk_B11(ik) * k**3 / (2.d0*pi**2) 
     &        * growth(zinit)**4 * 2.d0
         pk_B12(ik) = pk_B12(ik) * k**3 / (2.d0*pi**2) 
     &        * growth(zinit)**4 * 2.d0
         pk_B22(ik) = pk_B22(ik) * k**3 / (2.d0*pi**2) 
     &        * growth(zinit)**4 * 2.d0
c
ccccccccc
         write(6,'(i4, 1p8e18.10)') 
     &        ik, k, pk_A11(ik), pk_A12(ik), 
     &        pk_A22(ik), pk_B11(ik), pk_B12(ik), pk_B22(ik) 
ccccccccc
c
 9    continue
c
c     criteria for terminating integration (2)
c
      if((ik.gt.int(ik_max1/2)) .and. 
     &     (max(pk_A11(ik),pk_B11(ik)).le.1.d-20)) then
         ik_max2 = ik
         goto 100
      endif
c
      write(10,'(1p8e18.10)') 
     &     ak1(ik), growth(zred)**2*pk(ik), pk_A11(ik), pk_A12(ik), 
     &     pk_A22(ik), pk_B11(ik), pk_B12(ik), pk_B22(ik) 
c
 10   continue
c
 100  continue
c
      close(10)
c     
      end
c
c ******************************************************* c
c
      subroutine int_pk_B(x, zred, itype, intB11, intB12, intB22)
c
c ******************************************************* c
c
c     integral over mu by Gaussian quadratures method
c
      implicit none
      integer imu, imumax, itype
      parameter(imumax=10)
      real*8  x, zred, mumin, mumax, intB11, intB12, intB22
      real*8  pklin_mu, Lambda, lambda1, lambda2
      real*8  k, xmin, xmax 
      real*8  xmu(imumax), wmu(imumax)
      common /wave_number/  k, xmin, xmax
c
      intB11 = 0.0d0
      intB12 = 0.0d0
      intB22 = 0.0d0
c
      mumin = max(-1.0, (1.+x**2-xmax**2)/2./x)
      mumax = min( 1.0, (1.+x**2-xmin**2)/2./x)
      if(x.ge.0.5d0) mumax= 0.5d0/x
c
      call gauleg(mumin, mumax, xmu, wmu, imumax)
c 
      do imu=1, imumax
         call find_pk(k*sqrt(1.+x*x-2.*xmu(imu)*x), pklin_mu)
         lambda1 =  Lambda(1, itype, k, zred, x, xmu(imu))
         lambda2 =  Lambda(2, itype, k, zred, x, xmu(imu))
c
         intB11 = intB11 + wmu(imu) * x**2 * pklin_mu * lambda1**2
         intB12 = intB12 + wmu(imu) * x**2 * pklin_mu * lambda1*lambda2
         intB22 = intB22 + wmu(imu) * x**2 * pklin_mu * lambda2**2
      enddo
c
      intB11 = intB11 * x
      intB12 = intB12 * x
      intB22 = intB22 * x
c
      end
c
c ******************************************************* c
c
      function  Lambda(ib, itype, k, zred, x, mu)
c
c ******************************************************* c
c
c     computing the function Lambda by Gaussian quadratures
c
      implicit none
c
      integer ib, itype, iitype
      integer ieta, ieta_max
      parameter(ieta_max=20)
      real*8  k, k1, k2, k3, zred, x, mu, Lambda
      real*8  kk, zredinit, zzred, xx, mmu, Lambda_s
      real*8  zinit
      parameter(zinit=500.d0)
      real*8  ss, growth, eta_init, eta_fin
      real*8  int_lambda 
      real*8  eta_s1, eta_s2, Lambda_trapzd
      real*8  weta(ieta_max), xeta(ieta_max)
      common /integ_lambda/ zzred, kk, xx, mmu, zredinit, iitype
c     ---------------------------------------
c
      k1 = k
      k2 = k * x
      k3 = k * dsqrt(1.d0 - 2.d0*x*mu + x*x)
c
      if(max(k1,k2,k3).le.5.d-3) then 
c
c     using the linear propagator, integration is analytically performed
c
         ss = dlog(growth(zred)) - dlog(growth(zinit))
c
         if(ib.eq.1) then
            Lambda = 4.d0/35.d0 *x* (-1.d0 + mu*mu) * dexp(-1.5d0*ss)
     &         - 0.1d0 * (x + 5.d0*mu - 6.d0*x*mu*mu) * dexp(ss) 
     &          + 1.d0/14.d0 * (7.d0*mu + x * (3.d0 - 10.d0*mu*mu)) 
     &          * dexp(2.d0*ss)
         elseif(ib.eq.2) then
            Lambda = -6.d0/35.d0 *x* (-1.d0 + mu*mu) * dexp(-1.5d0*ss)
     &        - 0.1d0 * (x + 5.d0*mu - 6.d0*x*mu*mu) * dexp(ss) 
     &        + 1.d0/14.d0 * (7.d0*mu - x  - 6.d0*x*mu*mu) 
     &        * dexp(2.d0*ss)
         endif
c
         Lambda = Lambda / x / (1.d0 + x*x -2.d0*x*mu) 
c
      elseif(max(k1,k2,k3).gt.5.d-3) then 
c
         iitype = itype
         zredinit = zinit
         zzred = zred
         kk = k
         xx = x
         mmu = mu
c
         eta_init = dlog(growth(zinit))
         eta_fin  = dlog(growth(zred))
c
c     integral over "eta" by Gaussian quadratures 
c     
         Lambda = 0.d0
c
         call gauleg(eta_init, eta_fin, xeta, weta, ieta_max)
c
         do ieta=1, ieta_max
            Lambda = Lambda + weta(ieta) * int_lambda(ib, xeta(ieta))
         enddo
c
c     **** !! check !! **** 
c
c     integral over "eta" using trapezoidal formula 
c
c         Lambda_trapzd = 0.d0
c
c         eta_s1 = eta_init
c
c         do ieta=1, ieta_max-1
c
c            eta_s2 = eta_init + (eta_fin -eta_init) 
c     &           * dble(ieta)/dble(ieta_max-1)
c            Lambda_trapzd = Lambda_trapzd 
c     &        + (int_lambda(eta_s1) + int_lambda(eta_s2)) 
c     &           * (eta_s2 - eta_s1) /2.d0 
c            eta_s1 = eta_s2
c         enddo
c
c         write(6,*) 'k, Lambda, Lambda_trapzd=',k, Lambda, Lambda_trapzd
c
      endif
c
      end
c
c ******************************************************* c
c
      function int_lambda(ib, eta_s)
c
c ******************************************************* c
c
c     integrand of function Lambda, as function of eta_s
c
      implicit none
c
      integer ib, itype
      real*8 int_lambda, eta_s
      real*8 k, x, mu, zinit, zred, eta, eta0
      real*8 k1, k2, k3, growth
      real*8 G11a, G12a, G21a, G22a
      real*8 G11b, G12b, G21b, G22b
      real*8 G11c, G12c, G21c, G22c
      real*8 gamma112, gamma121, gamma222
      common /integ_lambda/ zred, k, x, mu, zinit, itype
c     ------------------------------------------
c
      eta  = dlog(growth(zred))
      eta0 = dlog(growth(zinit))
      k1 = k
      k2 = k * x
      k3 = k * dsqrt( 1.d0 - 2.d0*x*mu + x*x )
c
      if(itype.eq.0) then 
         call G_ab_RPT(k1, eta, eta_s, G11a, G12a, G21a, G22a)
         call G_ab_RPT(k2, eta_s, eta0, G11b, G12b, G21b, G22b)
         call G_ab_RPT(k3, eta_s, eta0, G11c, G12c, G21c, G22c)
      elseif(itype.eq.1) then
         call G_ab_Closure(k1, eta, eta_s, G11a, G12a, G21a, G22a)
         call G_ab_Closure(k2, eta_s, eta0, G11b, G12b, G21b, G22b)
         call G_ab_Closure(k3, eta_s, eta0, G11c, G12c, G21c, G22c)
      endif
c
      gamma112 = 0.5d0 * (1.d0 - x * mu) / (1.d0 - 2.d0*x*mu + x*x) 
      gamma121 = 0.5d0 * mu / x
      gamma222 = 0.5d0 * ( mu - x ) / (1.d0 - 2.d0*x*mu + x*x) / x
c
      if(ib.eq.1) then
         int_lambda = G11a * ( gamma112 * (G11b + G12b) * (G21c + G22c)
     &        + gamma121 * (G21b + G22b) * (G11c + G12c) )
     &        + G12a * gamma222 * (G21b + G22b) * (G21c + G22c) 
      elseif(ib.eq.2) then
         int_lambda = G21a * ( gamma112 * (G11b + G12b) * (G21c + G22c)
     &        + gamma121 * (G21b + G22b) * (G11c + G12c) )
     &        + G22a * gamma222 * (G21b + G22b) * (G21c + G22c)
      endif
c
      end
c
c ******************************************************* c
c
      subroutine search(itype,ik_init)
c
c ******************************************************* c
c
c     search the number record of last output data
c
      implicit none
c
      integer ik_init, ik, ikmax, itype, ialpha
      parameter(ikmax=5000)
      real*8 ak_dummy, dummy
      common /alpha_corr/ ialpha
c     ------------------------------------------
c
      if(itype.eq.0 .and. ialpha.eq.1) 
     &     open(10,file='renormalized_pk2_RPT.dat',status='unknown')
      if(itype.eq.0 .and. ialpha.eq.0) 
     &     open(10,file='renormalized_pk2_RPT_alpha.dat',
     &     status='unknown')
      if(itype.eq.1 .and. ialpha.eq.1) 
     &     open(10,file='renormalized_pk2_Closure.dat',status='unknown')
      if(itype.eq.1 .and. ialpha.eq.0) 
     & open(10,file='renormalized_pk2_Closure_alpha.dat',
     &     status='unknown')
c
      do ik=1, ikmax
         read(10,*,END=10) ak_dummy, dummy, dummy, dummy, dummy, 
     &         dummy, dummy, dummy
      enddo
c
 10   ik_init = ik
c
      write(6,*) ik_init, ak_dummy
c
      close(10)
c
      end
c
c ************************************************************
c
      SUBROUTINE gauleg(x1,x2,x,w,n)
c
c ************************************************************
c
      INTEGER n
      REAL*8 x1,x2,x(n),w(n)
      DOUBLE PRECISION EPS
      PARAMETER (EPS=3.d-14)
      INTEGER i,j,m
      DOUBLE PRECISION p1,p2,p3,pp,xl,xm,z,z1
      m=(n+1)/2
      xm=0.5d0*(x2+x1)
      xl=0.5d0*(x2-x1)
      do 12 i=1,m
        z=cos(3.141592654d0*(i-.25d0)/(n+.5d0))
1       continue
          p1=1.d0
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
11        continue
          pp=n*(z*p1-p2)/(z*z-1.d0)
          z1=z
          z=z1-p1/pp
        if(abs(z-z1).gt.EPS)goto 1
        x(i)=xm-xl*z
        x(n+1-i)=xm+xl*z
        w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
        w(n+1-i)=w(i)
12    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software z!0(0.
